==> CRTL-0.13.4-py3-none-any.whl: crtl/lkb/lkb.py <==
# Originally taken from https://github.com/allenai/scispacy/blob/main/scispacy/linking_utils.py
from collections import defaultdict
from dataclasses import asdict, dataclass
import gzip
import json
import logging
import pathlib
from typing import Dict, Iterable, List, Optional, Sequence, Set, Union
from spacy.util import registry

from crtl.s3_file_utils.s3_file_utils import cached_path


@dataclass(frozen=True)
class ConceptRelation:
    relation: str
    concept_id: Union[str, int]


@dataclass(frozen=True)
class Concept:
    concept_id: Union[str, int]
    canonical_name: str
    terms: frozenset[str]
    related_concepts: frozenset[ConceptRelation]
    types: frozenset[str]
    definition: Optional[str] = None

    @classmethod
    def from_json_dict(cls, concept_dict):
        concept_dict["related_concepts"] = frozenset(
            ConceptRelation(**relation_dict)
            for relation_dict in concept_dict["related_concepts"]
        )
        concept_dict["types"] = frozenset(concept_dict["types"])
        concept_dict["terms"] = frozenset(concept_dict["terms"])
        return cls(**concept_dict)

    def to_json_dict(self) -> Dict:
        concept_dict = asdict(self)
        concept_dict["related_concepts"] = list(
            asdict(concept_relation)
            for concept_relation in concept_dict["related_concepts"]
        )
        concept_dict["types"] = list(concept_dict["types"])
        concept_dict["terms"] = list(concept_dict["terms"])
        return concept_dict

    def __repr__(self):
        rep = ""
        num_terms = len(self.terms)
        rep = rep + f"CUI: {self.concept_id}, Name: {self.canonical_name}\n"
        rep = rep + f"Definition: {self.definition}\n"
        rep = rep + f"TUI(s): {', '.join(self.types)}\n"
        if num_terms > 10:
            rep = (
                rep
                + f"Terms (abbreviated, total: {num_terms}): \n\t {', '.join(list(self.terms)[:10])}"
            )
        else:
            rep = rep + f"Terms: (total: {num_terms}): \n\t {', '.join(self.terms)}"
        return rep


@registry.misc("lexical_knowledge_base_factory")
def lexical_knowledge_base_factory(concepts: Iterable[Concept]):
    return LexicalKnowledgeBase(concepts)


class LexicalKnowledgeBase:
    """
    A class representing two commonly needed views of a Lexical Knowledge Base:
    1. A mapping from concept_id to a Concept NamedTuple with more information.
    2. A mapping from terms to the sets of concept ids for which they are terms.
    Parameters
    ----------
    file_path: str, required.
        The file path to the jsonl representation of the KB to load.
    """

    def __init__(
        self,
        concepts: Iterable[Concept],
    ):
        term_to_cuis: Dict[str, Set[Union[str, int]]] = defaultdict(set)
        self.cui_to_concept: Dict[Union[str, int], Concept] = {}
        for concept in concepts:
            unique_terms = set(concept.terms)
            unique_terms.add(concept.canonical_name)
            for term in unique_terms:
                term_to_cuis[term].add(concept.concept_id)
            self.cui_to_concept[concept.concept_id] = concept
        self.term_to_cuis: Dict[str, Set[Union[str, int]]] = {**term_to_cuis}

    def to_disk(self, path):
        with open(path, "w") as f:
            for concept in self.cui_to_concept.values():
                json.dump(concept.to_json_dict(), f)
                f.write("\n")

    @classmethod
    def from_disk(cls, path):
        def concept_stream():
            with open(path) as f:
                for line in f:
                    yield Concept.from_json_dict(json.loads(line.strip()))

        return cls(concept_stream())

    @classmethod
    def from_gzipped_jsonl(cls, compressed_file_path):
        with gzip.open(compressed_file_path, "rb") as fin:
            data = fin.read().decode("utf-8").split("\n")

        def concept_stream():
            for line in data:
                if line:
                    yield Concept.from_json_dict(json.loads(line))

        return cls(concept_stream())


# @registry.readers("read_gzipped_jsonl")
def read_gzipped_jsonl(compressed_file_path: pathlib.Path) -> Iterable[Dict]:
    with gzip.open(compressed_file_path, "rb") as fin:
        data = fin.read().decode("utf-8").split("\n")
    for i, line in enumerate(data):
        try:
            yield json.loads(line)
        except:
            print(i, line)

@registry.readers("crtl.read_gzipped_jsonl_from_cached_path")
def make_gzipped_jsonl_from_cache_path_reader(compressed_file_path:Union[str,pathlib.Path]):
    compressed_file_path= cached_path(compressed_file_path)
    def read_gzipped_jsonl_from_cache_path() -> LexicalKnowledgeBase:
        return LexicalKnowledgeBase.from_gzipped_jsonl(compressed_file_path)
    return read_gzipped_jsonl_from_cache_path

==> CRTL-0.16.2-py3-none-any.whl: crtl/lkb/lkb.py <==
# Originally taken from https://github.com/allenai/scispacy/blob/main/scispacy/linking_utils.py
from collections import defaultdict
from dataclasses import asdict, dataclass
import gzip
import json
import logging
import pathlib
from typing import Dict, Iterable, List, Optional, Sequence, Set, Union
from spacy.util import registry

from crtl.s3_file_utils.s3_file_utils import cached_path


@dataclass(frozen=True)
# A dataclass to represent a relationship between concepts
class ConceptRelation:
    relation: str
    concept_id: Union[str, int]


# A dataclass to represent a Concept.
@dataclass(frozen=True)
class Concept:
    concept_id: Union[str, int]
    canonical_name: str
    terms: frozenset[str]
    related_concepts: frozenset[ConceptRelation]
    types: frozenset[str]
    definition: Optional[str] = None

    # when do you want to comparing two concepts???
    def __lt__(self, other):
        return self.concept_id < other.concept_id

    # A method to create a Concept object from a dictionary.
    @classmethod
    def from_json_dict(cls, concept_dict):
        concept_dict["related_concepts"] = frozenset(
            ConceptRelation(**relation_dict)
            for relation_dict in concept_dict["related_concepts"]
        )
        concept_dict["types"] = frozenset(concept_dict["types"])
        concept_dict["terms"] = frozenset(concept_dict["terms"])
        return cls(**concept_dict)

    # A method to convert a Concept object to a dictionary.
    def to_json_dict(self) -> Dict:
        concept_dict = asdict(self)
        concept_dict["related_concepts"] = list(
            asdict(concept_relation)
            for concept_relation in concept_dict["related_concepts"]
        )
        concept_dict["types"] = list(concept_dict["types"])
        concept_dict["terms"] = list(concept_dict["terms"])
        return concept_dict

    # A method to return a string representation of the Concept object.
    def __repr__(self):
        rep = ""
        num_terms = len(self.terms)
        rep = rep + f"CUI: {self.concept_id}, Name: {self.canonical_name}\n"
        rep = rep + f"Definition: {self.definition}\n"
        rep = rep + f"TUI(s): {', '.join(self.types)}\n"
        if num_terms > 10:
            rep = (
                rep
                + f"Terms (abbreviated, total: {num_terms}): \n\t {', '.join(list(self.terms)[:10])}"
            )
        else:
            rep = rep + f"Terms: (total: {num_terms}): \n\t {', '.join(self.terms)}"
        return rep


# A factory function that is registered with spaCy's registry.
@registry.misc("lexical_knowledge_base_factory")
def lexical_knowledge_base_factory(concepts: Iterable[Concept]):
    return LexicalKnowledgeBase(concepts)


# A class to represent a Lexical Knowledge Base.
class LexicalKnowledgeBase:
    """
    A class representing two commonly needed views of a Lexical Knowledge Base:
    1. A mapping from concept_id to a Concept NamedTuple with more information.
    2. A mapping from terms to the sets of concept ids for which they are terms.
    Parameters
    ----------
    file_path: str, required.
        The file path to the jsonl representation of the KB to load.
    """
    # Constructor which takes in iterable of Concepts and creates two dictionaries for mapping terms to concept IDs and vice versa
    def __init__(
        self,
        concepts: Iterable[Concept],
    ):
        # A mapping from terms to concept ids
        term_to_cuis: Dict[str, Set[Union[str, int]]] = defaultdict(set)
        # A mapping from concept_ids to Concepts
        self.cui_to_concept: Dict[Union[str, int], Concept] = {}
        for concept in concepts:
            unique_terms = set(concept.terms)
            unique_terms.add(concept.canonical_name)
            for term in unique_terms:
                term_to_cuis[term].add(concept.concept_id)
            self.cui_to_concept[concept.concept_id] = concept
        # Update self.term_to_cuis with the term_to_cuis dictionary
        self.term_to_cuis: Dict[str, Set[Union[str, int]]] = {**term_to_cuis}
    
    # Method to write this object to disk as a json file
    def to_disk(self, path):
        with open(path, "w") as f:
            for concept in self.cui_to_concept.values():
                json.dump(concept.to_json_dict(), f)
                f.write("\n")

    # Class method to create an instance of this class from a file on disk
    @classmethod
    def from_disk(cls, path):
        def concept_stream():
            with open(path) as f:
                for line in f:
                    yield Concept.from_json_dict(json.loads(line.strip()))

        return cls(concept_stream())

    # Class method to create an instance of this class from a gzipped jsonl file
    @classmethod
    def from_gzipped_jsonl(cls, compressed_file_path):
        with gzip.open(compressed_file_path, "rb") as fin:
            data = fin.read().decode("utf-8").split("\n")

        def concept_stream():
            for line in data:
                if line:
                    yield Concept.from_json_dict(json.loads(line))

        return cls(concept_stream())

# Function to read a gzipped jsonl file and return an iterable of dictionary objects
# @registry.readers("read_gzipped_jsonl")
def read_gzipped_jsonl(compressed_file_path: pathlib.Path) -> Iterable[Dict]:
    with gzip.open(compressed_file_path, "rb") as fin:
        data = fin.read().decode("utf-8").split("\n")
    for i, line in enumerate(data):
        try:
            yield json.loads(line)
        except:
            print(i, line)

# Spacy reader that reads gzipped jsonl file from a cached path and returns a LexicalKnowledgeBase instance
@registry.readers("crtl.read_gzipped_jsonl_from_cached_path")
def make_gzipped_jsonl_from_cache_path_reader(
    compressed_file_path: Union[str, pathlib.Path]
):
    compressed_file_path = cached_path(compressed_file_path)

    def read_gzipped_jsonl_from_cache_path() -> LexicalKnowledgeBase:
        return LexicalKnowledgeBase.from_gzipped_jsonl(compressed_file_path)

    return read_gzipped_jsonl_from_cache_path
